#zundler-iframe, #old-zundler-iframe {
    position:fixed;
    top:0; left:0; bottom:0; right:0;
    width:100%;
    height:100%;
    border:none;
    margin:0;
    padding:0;
    overflow:hidden;
    z-index:-1;
}

#loading-indicator {
  content: " ";
  z-index: 10;
  display: block;
  position: absolute;
  height: 100%;
  top: 0;
  left: 0;
  right: 0;
}

.lds-ripple {
  display: block;
  position: relative;
  width: 80px;
  height: 100%;
  margin: auto;
  z-index:1;
  -webkit-transform: translateY(50%);
  -ms-transform: translateY(50%);
  transform: translateY(50%);
}

.lds-ripple div {
  position: absolute;
  border: 4px solid #aaa;
  opacity: 1;
  border-radius: 50%;
  animation: lds-ripple 1s cubic-bezier(0, 0.2, 0.8, 1) infinite;
}

.lds-ripple div:nth-child(2) {
  animation-delay: -0.5s;
}

@keyframes lds-ripple {
  0% {
    top: 36px;
    left: 36px;
    width: 0;
    height: 0;
    opacity: 0;
  }
  4.9% {
    top: 36px;
    left: 36px;
    width: 0;
    height: 0;
    opacity: 0;
  }
  5% {
    top: 36px;
    left: 36px;
    width: 0;
    height: 0;
    opacity: 1;
  }
  100% {
    top: 0px;
    left: 0px;
    width: 72px;
    height: 72px;
    opacity: 0;
  }
}

.zundler-popup {
    display: none;
    position: fixed;
    left: 10%;
    top: 10%;
    width: 80%;
    height: 80%;
    background-color: white;
    border: 1px solid #ccc;
    z-index: 1000;
    box-shadow: 0 4px 8px rgba(0, 0, 0, 0.1);
    overflow: hidden;
    font-family: sans-serif;
}

.zundler-popup-sidebar {
    float: left;
    width: 20%;
    height: 100%;
    background-color: #f9f9f9;
    padding: 10px;
    box-sizing: border-box;
    border-right: 1px solid #ccc;
}


.zundler-popup-content {
    float: left;
    width: 80%;
    height: 100%;
    padding: 20px;
    box-sizing: border-box;
}

.content-frame {
    padding-bottom: 1.5em;
    height: 95%;
}

.content-section {
    overflow-y: auto;
    height: 100%;
    margin-top: 1.5em;
}

.zundler-popup-close-btn {
    position: absolute;
    top: 10px;
    right: 10px;
    border: none;
    background: none;
    font-size: 24px;
    cursor: pointer;
}
